package com.ycwl.basic.service.printer.impl;

import com.ycwl.basic.mapper.PrinterMapper;
import com.ycwl.basic.model.mobile.order.PriceObj;
import com.ycwl.basic.model.pc.price.entity.PriceConfigEntity;
import com.ycwl.basic.model.pc.printer.entity.PrintTaskEntity;
import com.ycwl.basic.model.pc.printer.entity.PrinterEntity;
import com.ycwl.basic.model.pc.printer.resp.MemberPrintResp;
import com.ycwl.basic.model.pc.printer.resp.PrinterResp;
import com.ycwl.basic.model.printer.req.PrinterSyncReq;
import com.ycwl.basic.model.printer.req.WorkerAuthReqVo;
import com.ycwl.basic.model.printer.resp.PrintTaskResp;
import com.ycwl.basic.repository.PriceRepository;
import com.ycwl.basic.service.printer.PrinterService;
import com.ycwl.basic.utils.ApiResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

@Service
public class PrinterServiceImpl implements PrinterService {
    @Autowired
    private PrinterMapper printerMapper;
    @Autowired
    private PriceRepository priceRepository;

    @Override
    public List<PrinterResp> listByScenicId(Long scenicId) {
        return printerMapper.listByScenicId(scenicId);
    }

    @Override
    public ApiResponse<List<PrinterEntity>> list(PrinterEntity condition) {
        return ApiResponse.success(printerMapper.list(condition));
    }

    @Override
    public ApiResponse<PrinterEntity> get(Integer id) {
        return ApiResponse.success(printerMapper.getById(id));
    }

    @Override
    public ApiResponse<Integer> add(PrinterEntity entity) {
        return ApiResponse.success(printerMapper.add(entity));
    }

    @Override
    public ApiResponse<Integer> update(PrinterEntity entity) {
        return ApiResponse.success(printerMapper.update(entity));
    }

    @Override
    public ApiResponse<Integer> delete(Integer id) {
        return ApiResponse.success(printerMapper.deleteById(id));
    }

    private PrinterEntity getPrinter(WorkerAuthReqVo req) {
        String accessKey = req.getAccessKey();
        if (accessKey == null) {
            return null;
        }
        PrinterEntity printer = printerMapper.findByAccessKey(accessKey);
        if (printer == null) {
            return null;
        }
        if (printer.getStatus() != 1) {
            return null;
        }
        return printer;
    }

    @Override
    public List<PrintTaskResp> sync(PrinterSyncReq req) {
        PrinterEntity printer = getPrinter(req);
        if (printer == null) {
            return Collections.emptyList();
        }
        List<String> printers = req.getPrinters();
        String printersStr = StringUtils.join(printers, ",");
        if (!StringUtils.equals(printersStr, printer.getPrinters())) {
            printer.setPrinters(printersStr);
            printerMapper.update(printer);
        }
        PrintTaskResp task = printerMapper.findTaskByPrinterId(printer.getId());
        if (task == null) {
            return Collections.emptyList();
        }
        return Collections.singletonList(task);
    }

    @Override
    public void taskSuccess(Integer taskId, WorkerAuthReqVo req) {
        PrintTaskEntity task = printerMapper.getTaskById(taskId);
        if (task == null) {
            return;
        }
        PrinterEntity printer = getPrinter(req);
        if (printer == null) {
            return;
        }
        if (printer.getId().equals(task.getPrinterId())) {
            printerMapper.updateTaskStatus(taskId, 1);
        }
    }

    @Override
    public void taskFail(Integer taskId, WorkerAuthReqVo req) {
        PrintTaskEntity task = printerMapper.getTaskById(taskId);
        if (task == null) {
            return;
        }
        PrinterEntity printer = getPrinter(req);
        if (printer == null) {
            return;
        }
        if (printer.getId().equals(task.getPrinterId())) {
            printerMapper.updateTaskStatus(taskId, 2);
        }
    }

    @Override
    public List<MemberPrintResp> getUserPhotoList(Long userId, Long scenicId) {
        List<MemberPrintResp> list = printerMapper.listRelation(userId, scenicId);
        if (list.isEmpty()) {
            // 额外逻辑
        }
        return list;
    }

    @Override
    public boolean deleteUserPhoto(Long memberId, Long scenicId, Long relationId) {
        int record = printerMapper.deleteUserPhoto(memberId, scenicId, relationId);
        return record > 0;
    }

    @Override
    public boolean addUserPhoto(Long memberId, Long scenicId, String url) {
        printerMapper.addUserPhoto(memberId, scenicId, url);
        return true;
    }

    @Override
    public MemberPrintResp getUserPhoto(Long userId, Long scenicId, Long id) {
        return printerMapper.getUserPhoto(userId, scenicId, id);
    }

    @Override
    public int setPhotoCropped(Long memberId, Long scenicId, Long id, String url) {
        return printerMapper.setPhotoCropped(memberId, scenicId, id, url);
    }

    @Override
    public int setPhotoQuantity(Long memberId, Long scenicId, Long id, Integer quantity) {
        return printerMapper.setPhotoQuantity(memberId, scenicId, id, quantity);
    }

    @Override
    public PriceObj queryPrice(Long memberId, Long scenicId) {
        List<MemberPrintResp> userPhotoList = getUserPhotoList(memberId, scenicId);
        // 判断几张
        PriceConfigEntity priceConfig = priceRepository.getPriceConfigByScenicTypeGoods(scenicId, 3, null);
        PriceObj obj = new PriceObj();
        obj.setPrice(priceConfig.getPrice().multiply(BigDecimal.valueOf(userPhotoList.size())));
        obj.setSlashPrice(priceConfig.getSlashPrice().multiply(BigDecimal.valueOf(userPhotoList.size())));
        obj.setGoodsType(3);
        obj.setFree(false);
        obj.setScenicId(scenicId);
        return obj;
    }
}